from .Decision_Tree import DecisionTree
from .KNN import knn
from .linear_regression import LinearRegression
from .multiple_regression import MultipleLinearRegression
from .Random_forest import RandomForest
from .SVM import svm
from .XGBoost import xgBoost
from .logistic_regression import LogisticRegression
from .Decision_Tree_Reg import DecisionTreeRegression